@@ -42,6 +42,9 @@
 #include "panel-lockdown.h"
 #include "panel-stock-icons.h"
 #include "panel-typebuiltins.h"
+#include "panel-icon-names.h"
+
+G_DEFINE_TYPE (PanelMenuBar, panel_menu_bar, GTK_TYPE_MENU_BAR);
 
 #define PANEL_MENU_BAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_MENU_BAR, PanelMenuBarPrivate))
 
@@ -62,51 +65,18 @@ enum {
 	PROP_ORIENTATION,
 };
 
-static GObjectClass *parent_class;
-
-static gboolean
-panel_menu_bar_enter_applications (GtkWidget        *widget,
-				   GdkEventCrossing *event,
-				   PanelMenuBar     *menubar)
-{
-	gtk_tooltips_set_tip (panel_tooltips,
-			      GTK_WIDGET (menubar),
-			      _("Browse and run installed applications"),
-			      NULL);
-	return FALSE;
-}
-
-static gboolean
-panel_menu_bar_enter_places (GtkWidget        *widget,
-			     GdkEventCrossing *event,
-			     PanelMenuBar     *menubar)
-{
-	gtk_tooltips_set_tip (panel_tooltips,
-			      GTK_WIDGET (menubar),
-			      _("Access documents, folders and network places"),
-			      NULL);
-	return FALSE;
-}
-
-static gboolean
-panel_menu_bar_enter_desktop (GtkWidget        *widget,
-			      GdkEventCrossing *event,
-			      PanelMenuBar     *menubar)
-{
-	gtk_tooltips_set_tip (panel_tooltips,
-			      GTK_WIDGET (menubar),
-			      _("Change desktop appearance and behavior, get help, or log out"),
-			      NULL);
-	return FALSE;
-}
+static void panel_menu_bar_update_text_gravity (PanelMenuBar *menubar);
 
 static gboolean
 panel_menu_bar_reinit_tooltip (GtkWidget    *widget,
 			       PanelMenuBar *menubar)
 {
-	gtk_tooltips_set_tip (panel_tooltips,
-			      GTK_WIDGET (menubar),
-			      "", NULL);
+	g_object_set (menubar->priv->applications_item,
+		      "has-tooltip", TRUE, NULL);
+	g_object_set (menubar->priv->places_item,
+		      "has-tooltip", TRUE, NULL);
+	g_object_set (menubar->priv->desktop_item,
+		      "has-tooltip", TRUE, NULL);
 
 	return FALSE;
 }
@@ -115,9 +85,7 @@ static gboolean
 panel_menu_bar_hide_tooltip (GtkWidget    *widget,
 			     PanelMenuBar *menubar)
 {
-	gtk_tooltips_set_tip (panel_tooltips,
-			      GTK_WIDGET (menubar),
-			      NULL, NULL);
+	g_object_set (widget, "has-tooltip", FALSE, NULL);
 
 	return FALSE;
 }
@@ -125,19 +93,12 @@ panel_menu_bar_hide_tooltip (GtkWidget    *widget,
 static void
 panel_menu_bar_setup_tooltip (PanelMenuBar *menubar)
 {
-	/* Update tooltip when we enter the menu items */
-	g_signal_connect (menubar->priv->applications_item,
-			  "enter-notify-event",
-			  G_CALLBACK (panel_menu_bar_enter_applications),
-			  menubar);
-	g_signal_connect (menubar->priv->places_item,
-			  "enter-notify-event",
-			  G_CALLBACK (panel_menu_bar_enter_places),
-			  menubar);
-	g_signal_connect (menubar->priv->desktop_item,
-			  "enter-notify-event",
-			  G_CALLBACK (panel_menu_bar_enter_desktop),
-			  menubar);
+	panel_util_set_tooltip_text (menubar->priv->applications_item,
+				     _("Browse and run installed applications"));
+	panel_util_set_tooltip_text (menubar->priv->places_item,
+				     _("Access documents, folders and network places"));
+	panel_util_set_tooltip_text (menubar->priv->desktop_item,
+				     _("Change desktop appearance and behavior, get help, or log out"));
 
 	//FIXME: this doesn't handle the right-click case. Sigh.
 	/* Hide tooltip if a menu is activated */
@@ -159,18 +120,10 @@ panel_menu_bar_setup_tooltip (PanelMenuBar *menubar)
 			  "deactivate",
 			  G_CALLBACK (panel_menu_bar_reinit_tooltip),
 			  menubar);
-
-	/* Preset a tooltip that will change when the cursor enters an item
-	 * If we don't do this, the tooltip will not work on the first enter
-	 * event. */
-	gtk_tooltips_set_tip (panel_tooltips,
-			      GTK_WIDGET (menubar),
-			      "", NULL);
 }
 
 static void
-panel_menu_bar_instance_init (PanelMenuBar      *menubar,
-			      PanelMenuBarClass *klass)
+panel_menu_bar_init (PanelMenuBar *menubar)
 {
 	GtkWidget *image;
 
@@ -181,7 +134,7 @@ panel_menu_bar_instance_init (PanelMenuBar      *menubar,
 	menubar->priv->applications_menu = create_applications_menu ("applications.menu", NULL);
 
 	menubar->priv->applications_item = gtk_image_menu_item_new_with_label (_("Applications"));
-	image = gtk_image_new_from_icon_name (PANEL_GNOME_LOGO_ICON,
+	image = gtk_image_new_from_icon_name (PANEL_ICON_MAIN_MENU,
 					      panel_menu_bar_icon_get_size ());
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menubar->priv->applications_item),
 				       image);
@@ -200,6 +153,11 @@ panel_menu_bar_instance_init (PanelMenuBar      *menubar,
 			       menubar->priv->desktop_item);
 
 	panel_menu_bar_setup_tooltip (menubar);
+
+	panel_menu_bar_update_text_gravity (menubar);
+	g_signal_connect (menubar, "screen-changed",
+			  G_CALLBACK (panel_menu_bar_update_text_gravity),
+			  NULL);
 }
 
 static void
@@ -280,7 +238,7 @@ panel_menu_bar_size_allocate (GtkWidget     *widget,
 	old_allocation.width  = widget->allocation.width;
 	old_allocation.height = widget->allocation.height;
 
-	GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
+	GTK_WIDGET_CLASS (panel_menu_bar_parent_class)->size_allocate (widget, allocation);
 
 	if (old_allocation.x      == allocation->x &&
 	    old_allocation.y      == allocation->y &&
@@ -303,8 +261,6 @@ panel_menu_bar_class_init (PanelMenuBarClass *klass)
 	GObjectClass   *gobject_class = (GObjectClass   *) klass;
 	GtkWidgetClass *widget_class  = (GtkWidgetClass *) klass;
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	gobject_class->get_property = panel_menu_bar_get_property;
         gobject_class->set_property = panel_menu_bar_set_property;
 
@@ -332,32 +288,6 @@ panel_menu_bar_class_init (PanelMenuBarClass *klass)
 		"class \"PanelMenuBar\" style \"panel-menubar-style\"");
 }
 
-GType
-panel_menu_bar_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		static const GTypeInfo info = {
-			sizeof (PanelMenuBarClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) panel_menu_bar_class_init,
-			NULL,
-			NULL,
-			sizeof (PanelMenuBar),
-			0,
-			(GInstanceInitFunc) panel_menu_bar_instance_init,
-			NULL
-		};
-
-		type = g_type_register_static (
-				GTK_TYPE_MENU_BAR, "PanelMenuBar", &info, 0);
-	}
-
-	return type;
-}
-
 static gboolean
 panel_menu_bar_on_expose (GtkWidget      *widget,
 			  GdkEventExpose *event,
@@ -468,18 +398,21 @@ panel_menu_bar_invoke_menu (PanelMenuBar *menubar,
 					   &error);
 		if (error) {
 			g_error_free (error);
+			error = NULL;
 			panel_launch_desktop_file ("gmenu-simple-editor.desktop",
 						   "gmenu-simple-editor",
 						   screen,
 						   &error);
 		}
 		if (error) {
-			panel_error_dialog (screen,
-					    "cannot_exec_gmenu-simple-editor", TRUE,
-					    _("Could not execute '%s'"),
-					    "%s",
-					    "gmenu-simple-editor",
-					    error->message);
+			char *primary;
+			primary = g_strdup_printf (_("Could not execute '%s'"),
+						   "gmenu-simple-editor");
+                        panel_error_dialog (NULL, screen,
+                                            "cannot_exec_gmenu-simple-editor",
+					    TRUE,
+					    primary, error->message);
+			g_free (primary);
 			g_error_free (error);
 		}
 	}
@@ -520,6 +453,27 @@ panel_menu_bar_change_background (PanelMenuBar *menubar)
 						      GTK_WIDGET (menubar));
 }
 
+static void
+set_item_text_gravity (GtkWidget *item)
+{
+	GtkWidget    *label;
+	PangoLayout  *layout;
+	PangoContext *context;
+
+	label = GTK_BIN (item)->child;
+	layout = gtk_label_get_layout (GTK_LABEL (label));
+	context = pango_layout_get_context (layout);
+	pango_context_set_base_gravity (context, PANGO_GRAVITY_AUTO);
+}
+
+static void
+panel_menu_bar_update_text_gravity (PanelMenuBar *menubar)
+{
+	set_item_text_gravity (menubar->priv->applications_item);
+	set_item_text_gravity (menubar->priv->places_item);
+	set_item_text_gravity (menubar->priv->desktop_item);
+}
+
 static void
 set_item_text_angle_and_alignment (GtkWidget *item,
 				   double     text_angle,
