@@ -27,45 +27,47 @@
 
 #include <string.h>
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#include <gio/gio.h>
 #include "menu.h"
 #include "panel-util.h"
 #include "panel-globals.h"
 #include "panel-recent.h"
 #include "panel-stock-icons.h"
 #include "panel-multiscreen.h"
+#include "panel-icon-names.h"
 
 static gboolean
 show_uri (const char *uri, const char *mime_type, GdkScreen *screen,
 	  GError **error)
 {
 	char **env;
-	GnomeVFSResult result;
-	GnomeVFSMimeApplication *app;
+	GFile *file;
+	GAppInfo *app;
 	GList *uris = NULL;
+	gboolean ret;
+
+	file = g_file_new_for_uri (uri);
+	app = g_app_info_get_default_for_type (mime_type, !g_file_is_native (file));
+	g_object_unref (file);
 
-	app = gnome_vfs_mime_get_default_application_for_uri (uri, mime_type);
 	if (app == NULL) {
-		g_set_error (error, 0, 0, _("Could not find a suitable application."));
+		g_set_error (error, 0, 0,
+			     _("Could not find a suitable application."));
 		return FALSE;
 	}
 
 	env = panel_make_environment_for_screen (screen, NULL);
 
+	//FIXME: would g_app_info_launch_default_for_uri() be enough?
 	uris = g_list_append (uris, (gpointer)uri);
-	result = gnome_vfs_mime_application_launch_with_env (app, uris, env);
+	//FIXME: use GdkAppLaunchContext
+	ret = g_app_info_launch_uris (app, uris, NULL, error);
 	g_list_free (uris);
 
 	g_strfreev (env);
-	gnome_vfs_mime_application_free (app);
+	g_object_unref (app);
 
-	if (result != GNOME_VFS_OK) {
-		g_set_error (error, 0, 0, gnome_vfs_result_to_string (result));
-		return FALSE;
-	}
-
-	return TRUE;
+	return ret;
 }
 
 
@@ -210,10 +212,9 @@ panel_recent_append_documents_menu (GtkWidget        *top_menu,
 	menu_item = gtk_image_menu_item_new ();
 	setup_menu_item_with_icon (menu_item,
 				   panel_menu_icon_get_size (),
-				   NULL,
-				   GTK_STOCK_OPEN,
-				   _("Recent Documents"),
-				   TRUE);
+				   PANEL_ICON_RECENT,
+				   NULL, NULL,
+				   _("Recent Documents"));
 	recent_menu = gtk_recent_chooser_menu_new_for_manager (manager);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), recent_menu);
 
@@ -251,13 +252,10 @@ panel_recent_append_documents_menu (GtkWidget        *top_menu,
 	setup_menu_item_with_icon (menu_item,
 				   panel_menu_icon_get_size (),
 				   NULL,
-				   GTK_STOCK_CLEAR,
-				   _("Clear Recent Documents"),
-				   TRUE);
-	gtk_tooltips_set_tip (panel_tooltips,
-			      menu_item,
-			      _("Clear all items from the recent documents list"),
-			      NULL);
+				   GTK_STOCK_CLEAR, NULL,
+				   _("Clear Recent Documents"));
+	panel_util_set_tooltip_text (menu_item,
+				     _("Clear all items from the recent documents list"));
 	gtk_menu_shell_append (GTK_MENU_SHELL (recent_menu), menu_item);
 
 	g_signal_connect (menu_item, "activate",
