@@ -36,6 +36,12 @@ void
 panel_session_request_logout (void)
 {
 	GnomeClient *client;
+	static int   recursion_guard = 0;
+
+	if (recursion_guard)
+		return;
+
+	recursion_guard++;
 
 	if (!(client = gnome_master_client ()))
 		return;
@@ -48,8 +54,10 @@ panel_session_request_logout (void)
 				   GNOME_SAVE_GLOBAL,
 				   TRUE,
 				   GNOME_INTERACT_ANY,
-				   FALSE,
+				   TRUE, /* do not use the gnome-session gui */
 				   TRUE);
+
+	recursion_guard--;
 }
 
 static void
@@ -69,6 +77,16 @@ panel_session_handle_die_request (GnomeClient *client)
 	gtk_main_quit ();
 }
 
+void
+panel_session_do_not_restart (void)
+{
+	GnomeClient *client;
+
+	client = gnome_master_client ();
+
+	gnome_client_set_restart_style (client, GNOME_RESTART_IF_RUNNING);
+}
+
 void
 panel_session_init (void)
 {
